@@ -5,13 +5,16 @@
 // This file is licensed under the GPLv2 or later
 //
 // Copyright (C) 2009 Koji Otani <sho@bbr.jp>
-// Copyright (C) 2009 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2009, 2010 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2010 Carlos Garcia Campos <carlosgc@gnome.org>
 //
 //========================================================================
 
 #ifndef POPPLER_CACHE_H
 #define POPPLER_CACHE_H
 
+#include "Object.h"
+
 class PopplerCacheItem
 {
   public:
@@ -58,4 +61,18 @@ class PopplerCache
     int cacheSize;
 };
 
+class PopplerObjectCache
+{
+  public:
+    PopplerObjectCache (int cacheSizeA, XRef *xrefA);
+    ~PopplerObjectCache();
+
+    Object *put(const Ref &ref);
+    Object *lookup(const Ref &ref, Object *obj);
+
+  private:
+    XRef *xref;
+    PopplerCache *cache;
+};
+
 #endif
